/*
 * YouGrabber
 *
 * A lightweight, ncurses-based, multi-threaded command line YouTube.com
 * video downloader with proxy support, made from scratch in ANSI C.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "common.h"

static const unsigned short __FILE_MAX = 999;
static const char __VALID_CH[] = "._-";
static const char __SPACE_CH = '_';

/* downloads in progress */
static unsigned __DIP = 0;

/* set dip (downloads in progress) counter */
void dip_set(int i)
{
  __DIP = i;
}

/* remove a download from the dip counter */
void dip_rem(void)
{
  __DIP--;
}

/* return current dip counter value */
int dip_get(void)
{
  return __DIP;
}

/* file name generator */
char *file_name_gen(char *dest, char *name, char *ext)
{
  char tmp[strlen(name)];
  unsigned i;
  unsigned p;

  for (i = p = 0; i < strlen(name); ++i)
    tmp[p++] = (isalnum(name[i]) || strchr(__VALID_CH, name[i]))
      ? name[i] : __SPACE_CH;

  tmp[p] = 0;

  for (i = 0; i <= __FILE_MAX; ++i)
  {
    if (i)
      sprintf(dest, "%s_%.3d.%s", tmp, i, ext);
    else
      sprintf(dest, "%s.%s", tmp, ext);

    if (access(dest, F_OK))
      return dest;
  }

  return dest = NULL;
}

/* cut a string between two delimiters */
char *strcut(char *str, char *d1, char *d2)
{
  unsigned cut_start = 0;
  unsigned cut_stop = 0;
  unsigned i;
  unsigned j;

  for (i = j = 0; i < strlen(str); ++i)
  {
    if (!cut_start)
    {
      j = (str[i] == d1[j]) ? j+1 : 0;

      if (j == strlen(d1))
      {
        cut_start = i;
        j = 0;
      }
    }

    if (cut_start)
    {
      j = (str[i] == d2[j]) ? j+1 : 0;

      if (j == strlen(d2))
        cut_stop = (i+1)-strlen(d2);
    }

    if (cut_start && cut_stop)
      break;
  }

  if (!cut_start || !cut_stop)
    return str;

  char buf[cut_stop-cut_start];

  for (i = cut_start+1, j = 0; i < cut_stop; ++i, ++j)
    buf[j] = str[i];

  buf[j] = 0;

  return strcpy(str, buf);
}
